
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.008.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.4220"
  },
  "configure_requires" => {
    "ExtUtils::MakeMaker" => "7.24",
    "Module::Build" => "0.4220"
  },
  "dist_abstract" => "An asynchronous, non-blocking shadowsocks client and server module.",
  "dist_author" => [
    "Li ZHOU <lzh\@cpan.org>"
  ],
  "dist_name" => "Net-Shadowsocks",
  "dist_version" => "0.7.0",
  "license" => "perl",
  "module_name" => "Net::Shadowsocks",
  "recursive_test_files" => 1,
  "requires" => {
    "AnyEvent" => "7.13",
    "AnyEvent::Handle" => 0,
    "AnyEvent::Socket" => 0,
    "Carp" => "1.40",
    "Crypt::Cipher::RC6" => "0.044",
    "Crypt::Mode::CBC" => "0.044",
    "Crypt::Random" => "1.25",
    "Digest::MD5" => "2.55",
    "Digest::SHA" => "5.96",
    "IO::Socket::Socks" => "0.73",
    "Mcrypt" => "v2.5.7.0",
    "Socket" => "2.024",
    "perl" => "5.006",
    "strict" => 0,
    "warnings" => 0
  },
  "test_requires" => {
    "ExtUtils::MakeMaker" => "7.24",
    "File::Spec" => "3.62",
    "IO::Handle" => "1.36",
    "IPC::Open3" => "1.20",
    "Test::CheckDeps" => "0.010",
    "Test::More" => "1.302062",
    "blib" => "1.06"
  }
);


my %fallback_build_requires = (
  "ExtUtils::MakeMaker" => "7.24",
  "File::Spec" => "3.62",
  "IO::Handle" => "1.36",
  "IPC::Open3" => "1.20",
  "Module::Build" => "0.4220",
  "Test::CheckDeps" => "0.010",
  "Test::More" => "1.302062",
  "blib" => "1.06"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
